﻿using Borepin.Service;
using FabAccessAPI;
using Prism.Navigation;
using Prism.Services;
using System.Threading.Tasks;

namespace Borepin.Base
{
    /// <summary>
    /// Base for all BFFH Based PageModels
    /// </summary>
    public abstract class ConnectionModelBase : PageModelBase
    {
        #region Private Fields
        protected readonly IPageDialogService _PageDialogService;
        protected readonly IAPI _API;
        #endregion

        #region Constructors
        protected ConnectionModelBase(INavigationService navigationService, IPageDialogService pageDialogService, IAPIService apiService) : base(navigationService)
        {
            _PageDialogService = pageDialogService;
            _API = apiService.GetAPI();
            _API.ConnectionStatusChanged += OnConnectionStatusChanged;

            IsConnected = _API.IsConnected;
        }
        #endregion

        #region Methods
        public void OnConnectionStatusChanged(object sender, ConnectionStatusChange args)
        {
            switch(args)
            {
                case ConnectionStatusChange.Connected:
                    IsConnected = true;
                    LoadAPIData();
                    break;
                case ConnectionStatusChange.Reconnected:
                    ReloadAPIData();
                    break;
                case ConnectionStatusChange.ConnectionLoss:
                case ConnectionStatusChange.Disconnected:
                    IsConnected = false;
                    break;
            }
        }

        public virtual Task LoadAPIData()
        {
            return Task.CompletedTask;
        }
        public virtual Task ReloadAPIData()
        {
            return Task.CompletedTask;
        }

        public virtual Task LoadInstance(object instance)
        {
            return Task.CompletedTask;
        }

        public virtual Task<object> CreateInstance()
        {
            return Task.FromResult<object>(null);
        }
        #endregion

        #region Fields
        /// <summary>
        /// PageModel is Connected
        /// </summary>
        private bool _IsConnected = true;
        public bool IsConnected
        {
            get => _IsConnected;
            set => SetProperty(ref _IsConnected, value);
        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedTo(INavigationParameters parameters)
        {
            if(parameters.ContainsKey("instance"))
            {
                LoadInstance(parameters.GetValue<object>("instance"));
            }
            else
            {
                Log.Trace("No instance");
                LoadInstance(null);
            }

            if(_API.IsConnected)
            {
                LoadAPIData();
            }
        }

        public override void OnNavigatedFrom(INavigationParameters parameters)
        {
            object instance = CreateInstance();
            if(instance != null)
            {
                parameters.Add("instance", instance);
            }
        }
        #endregion
    }
}